@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2019, OFFIS e.V.
+ *  Copyright (C) 1994-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -22,10 +22,6 @@
 
 #include "dcmtk/config/osconfig.h"    /* make sure OS specific configuration is included first */
 
-#define INCLUDE_CSTDLIB
-#define INCLUDE_CSTDIO
-#include "dcmtk/ofstd/ofstdinc.h"
-
 #include "dcmtk/ofstd/ofstream.h"
 #include "dcmtk/ofstd/ofuuid.h"
 
@@ -210,11 +206,11 @@ OFCondition DcmPixelSequence::makeSubObject(DcmObject *&subObject,
     switch (newTag.getEVR())
     {
         case EVR_na:
-            if (newTag.getXTag() == DCM_Item)
+            if (newTag == DCM_Item)
                 newObject = new DcmPixelItem(newTag, newLength);
-            else if (newTag.getXTag() == DCM_SequenceDelimitationItem)
+            else if (newTag == DCM_SequenceDelimitationItem)
                 l_error = EC_SequEnd;
-            else if (newTag.getXTag() == DCM_ItemDelimitationItem)
+            else if (newTag == DCM_ItemDelimitationItem)
                 l_error = EC_ItemEnd;
             else
                 l_error = EC_InvalidTag;
