@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2017, OFFIS e.V.
+ *  Copyright (C) 1994-2018, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -54,14 +54,48 @@ DcmShortText &DcmShortText::operator=(const DcmShortText &obj)
 }
 
 
+int DcmShortText::compare(const DcmElement& rhs) const
+{
+    int result = DcmElement::compare(rhs);
+    if (result != 0)
+    {
+        return result;
+    }
+
+    /* cast away constness (dcmdata is not const correct...) */
+    DcmShortText* myThis = NULL;
+    DcmShortText* myRhs = NULL;
+    myThis = OFconst_cast(DcmShortText*, this);
+    myRhs = OFstatic_cast(DcmShortText*, OFconst_cast(DcmElement*, &rhs));
+
+    /* compare length */
+    unsigned long thisLength = myThis->getLength();
+    unsigned long rhsLength = myRhs->getLength();
+    if (thisLength < rhsLength)
+    {
+        return -1;
+    }
+    else if (thisLength > rhsLength)
+    {
+        return 1;
+    }
+
+    /* check whether values are equal */
+    OFString thisValue, rhsValue;
+    myThis->getOFStringArray(thisValue);
+    myThis->getOFStringArray(rhsValue);
+    return thisValue.compare(rhsValue);
+}
+
+
 OFCondition DcmShortText::copyFrom(const DcmObject& rhs)
 {
-  if (this != &rhs)
-  {
-    if (rhs.ident() != ident()) return EC_IllegalCall;
-    *this = OFstatic_cast(const DcmShortText &, rhs);
-  }
-  return EC_Normal;
+    if (this != &rhs)
+    {
+        if (rhs.ident() != ident()) return EC_IllegalCall;
+        *this = OFstatic_cast(const DcmShortText &, rhs);
+    }
+    return EC_Normal;
 }
 
 
