@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1996-2010, OFFIS e.V.
+ *  Copyright (C) 1996-2022, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -37,9 +37,6 @@
 #include "dcmtk/dcmimgle/diplugin.h"
 #include "dcmtk/dcmdata/dcdicent.h"  /* needed by MSVC5 */
 
-#define INCLUDE_CCTYPE
-#include "dcmtk/ofstd/ofstdinc.h"
-
 #ifndef FILENAME_MAX
 #define FILENAME_MAX 255
 #endif
@@ -203,12 +200,12 @@ void DicomImage::Init()
             if (cstr != NULL)
             {
                 char *q = cstr;
-                unsigned char c;
+                char c;
                 for (const char *p = str; *p != 0; p++)    // remove invalid chars
                 {
-                    c = OFstatic_cast(unsigned char, *p);
+                    c = *p;
                     if (isalpha(c))
-                        *(q++) = toupper(c);
+                        *(q++) = OFstatic_cast(char, toupper(c));
                     else if (isdigit(c))
                         *(q++) = c;
                 }
@@ -359,7 +356,7 @@ DicomImage *DicomImage::createDicomImage(unsigned long fstart,
 
 
 // --- create scaled to given size ('width' and 'height') image, memory isn't handled internally !
-// --- if one dimension ist '0' the other is automatically adjusted (with respect to pixel aspect ratio)
+// --- if one dimension is '0' the other is automatically adjusted (with respect to pixel aspect ratio)
 
 DicomImage *DicomImage::createScaledImage(const unsigned long width,
                                           const unsigned long height,
@@ -760,7 +757,7 @@ int DicomImage::writeBMP(FILE *stream,
 }
 
 
-// --- write 'frame' of image data to 'filename' plugable image format
+// --- write 'frame' of image data to 'filename' pluggable image format
 
 int DicomImage::writePluginFormat(const DiPluginFormat *plugin,
                                   const char *filename,
@@ -783,7 +780,7 @@ int DicomImage::writePluginFormat(const DiPluginFormat *plugin,
 }
 
 
-// --- same for open C 'FILE' in plugable image format
+// --- same for open C 'FILE' in pluggable image format
 
 int DicomImage::writePluginFormat(const DiPluginFormat *plugin,
                                   FILE *stream,
