@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1998-2014, OFFIS e.V.
+ *  Copyright (C) 1998-2021, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -25,11 +25,6 @@
 #include "dcmtk/dcmimgle/diciefn.h"
 #include "dcmtk/ofstd/ofstream.h"
 
-#define INCLUDE_CMATH
-#define INCLUDE_CSTRING
-#include "dcmtk/ofstd/ofstdinc.h"
-
-
 /*----------------*
  *  constructors  *
  *----------------*/
