@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2001-2014, OFFIS e.V.
+ *  Copyright (C) 2001-2021, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -23,11 +23,9 @@
 #include "dcmtk/dcmjpeg/djdijg12.h"
 #include "dcmtk/dcmjpeg/djcparam.h"
 #include "dcmtk/dcmdata/dcerror.h"
-
-#define INCLUDE_CSTDIO
-#define INCLUDE_CSETJMP
-#define INCLUDE_CSTRING
 #include "dcmtk/ofstd/ofstdinc.h"
+#include "dcmtk/ofstd/ofdiag.h"
+#include <csetjmp>
 
 // These two macros are re-defined in the IJG header files.
 // We undefine them here and hope that IJG's configure has
@@ -50,11 +48,12 @@ BEGIN_EXTERN_C
 #undef const
 #endif
 
-#ifdef USE_STD_CXX_INCLUDES
 // Solaris defines longjmp() in namespace std, other compilers don't...
-namespace std { }
-using namespace std;
-#endif
+using STD_NAMESPACE longjmp;
+using STD_NAMESPACE jmp_buf;
+
+#include DCMTK_DIAGNOSTIC_PUSH
+#include DCMTK_DIAGNOSTIC_IGNORE_VISUAL_STUDIO_DECLSPEC_PADDING_WARNING
 
 // private error handler struct
 struct DJDIJG12ErrorStruct
@@ -69,6 +68,8 @@ struct DJDIJG12ErrorStruct
   DJDecompressIJG12Bit *instance;
 };
 
+#include DCMTK_DIAGNOSTIC_POP
+
 // private source manager struct
 struct DJDIJG12SourceManagerStruct
 {
@@ -202,6 +203,8 @@ DJDecompressIJG12Bit::~DJDecompressIJG12Bit()
   cleanup();
 }
 
+#include DCMTK_DIAGNOSTIC_PUSH
+#include DCMTK_DIAGNOSTIC_IGNORE_VISUAL_STUDIO_OBJECT_DESTRUCTION_WARNING
 
 OFCondition DJDecompressIJG12Bit::init()
 {
@@ -449,6 +452,8 @@ OFCondition DJDecompressIJG12Bit::decode(
   return EC_Normal;
 }
 
+#include DCMTK_DIAGNOSTIC_POP
+
 void DJDecompressIJG12Bit::emitMessage(int msg_level) const
 {
   // This is how we map the message levels:
