@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1997-2018, OFFIS e.V.
+ *  Copyright (C) 1997-2021, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -23,11 +23,9 @@
 #include "dcmtk/dcmjpeg/djeijg12.h"
 #include "dcmtk/dcmjpeg/djcparam.h"
 #include "dcmtk/dcmdata/dcerror.h"
-
-#define INCLUDE_CSTDIO
-#define INCLUDE_CSETJMP
-#define INCLUDE_CSTRING
 #include "dcmtk/ofstd/ofstdinc.h"
+#include "dcmtk/ofstd/ofdiag.h"
+#include <csetjmp>
 
 // These two macros are re-defined in the IJG header files.
 // We undefine them here and hope that IJG's configure has
@@ -54,11 +52,12 @@ BEGIN_EXTERN_C
 #undef const
 #endif
 
-#ifdef USE_STD_CXX_INCLUDES
 // Solaris defines longjmp() in namespace std, other compilers don't...
-namespace std { }
-using namespace std;
-#endif
+using STD_NAMESPACE longjmp;
+using STD_NAMESPACE jmp_buf;
+
+#include DCMTK_DIAGNOSTIC_PUSH
+#include DCMTK_DIAGNOSTIC_IGNORE_VISUAL_STUDIO_DECLSPEC_PADDING_WARNING
 
 // private error handler struct
 struct DJEIJG12ErrorStruct
@@ -73,6 +72,8 @@ struct DJEIJG12ErrorStruct
   DJCompressIJG12Bit *instance;
 };
 
+#include DCMTK_DIAGNOSTIC_POP
+
 // callback forward declarations
 void DJEIJG12ErrorExit(j_common_ptr);
 void DJEIJG12EmitMessage(j_common_ptr cinfo, int msg_level);
@@ -336,6 +337,9 @@ OFCondition DJCompressIJG12Bit::encode(
   return EC_IllegalCall;
 }
 
+#include DCMTK_DIAGNOSTIC_PUSH
+#include DCMTK_DIAGNOSTIC_IGNORE_VISUAL_STUDIO_OBJECT_DESTRUCTION_WARNING
+
 OFCondition DJCompressIJG12Bit::encode(
   Uint16 columns,
   Uint16 rows,
@@ -488,6 +492,8 @@ OFCondition DJCompressIJG12Bit::encode(
   return EC_Normal;
 }
 
+#include DCMTK_DIAGNOSTIC_POP
+
 void DJCompressIJG12Bit::initDestination(jpeg_compress_struct *cinfo)
 {
   cleanup(); // erase old list of compressed blocks, if any
