@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1996-2019, OFFIS e.V.
+ *  Copyright (C) 1996-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -21,12 +21,6 @@
 
 #include "dcmtk/config/osconfig.h" /* make sure OS specific configuration is included first */
 
-#define INCLUDE_CSTDLIB
-#define INCLUDE_CSTDIO
-#define INCLUDE_CSTRING
-#define INCLUDE_CCTYPE
-#include "dcmtk/ofstd/ofstdinc.h"
-
 BEGIN_EXTERN_C
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
@@ -53,6 +47,9 @@ END_EXTERN_C
 #include "dcmtk/dcmdata/dcuid.h"     /* for dcmtk version name */
 #include "dcmtk/dcmdata/dcostrmz.h"  /* for dcmZlibCompressionLevel */
 #include "dcmtk/dcmtls/tlsopt.h"     /* for DcmTLSOptions */
+#include "dcmtk/ofstd/ofstdinc.h"
+#include <ctime>
+
 
 #ifdef ON_THE_FLY_COMPRESSION
 #include "dcmtk/dcmjpeg/djdecode.h"  /* for JPEG decoders */
@@ -108,10 +105,10 @@ static OFCmdUnsignedInt opt_inventStudyCount = 50;
 static OFCmdUnsignedInt opt_inventSeriesCount = 100;
 static OFBool opt_inventSOPInstanceInformation = OFFalse;
 static OFBool opt_correctUIDPadding = OFFalse;
-static OFString patientNamePrefix("OFFIS^TEST_PN_");   // PatientName is PN (maximum 16 chars)
-static OFString patientIDPrefix("PID_"); // PatientID is LO (maximum 64 chars)
-static OFString studyIDPrefix("SID_");   // StudyID is SH (maximum 16 chars)
-static OFString accessionNumberPrefix;   // AccessionNumber is SH (maximum 16 chars)
+static OFString patientNamePrefix("OFFIS^TEST_PN_");  // PatientName is PN (maximum 16 chars)
+static OFString patientIDPrefix("PID_");              // PatientID is LO (maximum 64 chars)
+static OFString studyIDPrefix("SID_");                // StudyID is SH (maximum 16 chars)
+static OFString accessionNumberPrefix;                // AccessionNumber is SH (maximum 16 chars)
 static const char *opt_configFile = NULL;
 static const char *opt_profileName = NULL;
 T_DIMSE_BlockingMode opt_blockMode = DIMSE_BLOCKING;
@@ -128,6 +125,9 @@ static T_ASC_UserIdentityNegotiationMode opt_identMode = ASC_USER_IDENTITY_NONE;
 static OFString opt_user;
 static OFString opt_password;
 static OFString opt_identFile;
+// Denotes whether we ask for an explicit server response for
+// our identity negotiation request and only continue if this
+// positive response is provided by the SCP.
 static OFBool opt_identResponse = OFFalse;
 
 static OFCondition
@@ -440,7 +440,7 @@ int main(int argc, char *argv[])
         app.checkValue(cmd.getValue(opt_profileName));
 
         // read configuration file. The profile name is checked later.
-        OFCondition cond = DcmAssociationConfigurationFile::initialize(asccfg, opt_configFile);
+        OFCondition cond = DcmAssociationConfigurationFile::initialize(asccfg, opt_configFile, OFTrue);
         if (cond.bad())
         {
           OFLOG_ERROR(storescuLogger, "reading config file: " << cond.text());
@@ -558,13 +558,13 @@ int main(int argc, char *argv[])
       if (cmd.findOption("--empty-password"))
       {
         app.checkDependence("--empty-password", "--user", opt_identMode == ASC_USER_IDENTITY_USER);
-        opt_password= "";
+        opt_password = "";
         opt_identMode = ASC_USER_IDENTITY_USER_PASSWORD;
       }
       cmd.endOptionBlock();
       if (cmd.findOption("--pos-response"))
       {
-         app.checkDependence("--pos-response", "--user, --kerberos or --saml", opt_identMode != ASC_USER_IDENTITY_NONE);
+         app.checkDependence("--pos-response", "--user, --kerberos, --saml or --jwt", opt_identMode != ASC_USER_IDENTITY_NONE);
          opt_identResponse = OFTrue;
       }
    }
@@ -704,7 +704,7 @@ int main(int argc, char *argv[])
     }
 
     /* initialize association parameters, i.e. create an instance of T_ASC_Parameters*. */
-    cond = ASC_createAssociationParameters(&params, opt_maxReceivePDULength);
+    cond = ASC_createAssociationParameters(&params, opt_maxReceivePDULength, dcmConnectionTimeout.get());
     if (cond.bad()) {
       OFLOG_FATAL(storescuLogger, DimseCondition::dump(temp_str, cond));
       return 1;
@@ -783,9 +783,6 @@ int main(int argc, char *argv[])
       }
     }
 
-    /* dump the connection parameters if in debug mode*/
-    OFLOG_DEBUG(storescuLogger, ASC_dumpConnectionParameters(temp_str, assoc));
-
     /* dump the presentation contexts which have been accepted/refused */
     if (opt_showPresentationContexts)
       OFLOG_INFO(storescuLogger, "Association Parameters Negotiated:" << OFendl << ASC_dumpParameters(temp_str, params, ASC_ASSOC_AC));
@@ -863,8 +860,8 @@ int main(int argc, char *argv[])
       cond = ASC_abortAssociation(assoc);
       if (cond.bad()) {
         OFLOG_ERROR(storescuLogger, "Association Abort Failed: " << DimseCondition::dump(temp_str, cond));
-        return 1;
       }
+      return 1;
     }
 
     /* destroy the association, i.e. free memory of T_ASC_Association* structure. This */
@@ -894,7 +891,7 @@ int main(int argc, char *argv[])
     if (opt_haltOnUnsuccessfulStore && unsuccessfulStoreEncountered) {
       if (lastStatusCode == STATUS_Success) {
         // there must have been some kind of general network error
-        exitCode = 0xff;
+        exitCode = 10;
       } else {
         exitCode = (lastStatusCode >> 8); // only the least significant byte is relevant as exit code
       }
@@ -944,7 +941,7 @@ addPresentationContext(T_ASC_Parameters *params,
   T_ASC_SC_ROLE proposedRole = ASC_SC_ROLE_DEFAULT)
 {
   const char *c_p = transferSyntax.c_str();
-  OFCondition cond = ASC_addPresentationContext(params, presentationContextId,
+  OFCondition cond = ASC_addPresentationContext(params, OFstatic_cast(T_ASC_PresentationContextID, presentationContextId),
     abstractSyntax.c_str(), &c_p, 1, proposedRole);
   return cond;
 }
@@ -966,7 +963,7 @@ addPresentationContext(T_ASC_Parameters *params,
     ++s_cur;
   }
 
-  OFCondition cond = ASC_addPresentationContext(params, presentationContextId,
+  OFCondition cond = ASC_addPresentationContext(params, OFstatic_cast(T_ASC_PresentationContextID, presentationContextId),
     abstractSyntax.c_str(), transferSyntaxes, transferSyntaxCount, proposedRole);
 
   delete[] transferSyntaxes;
@@ -1240,7 +1237,7 @@ progressCallback(void * /*callbackData*/,
   if (progress->state == DIMSE_StoreBegin)
   {
     OFString str;
-    OFLOG_DEBUG(storescuLogger, DIMSE_dumpMessage(str, *req, DIMSE_OUTGOING));
+    OFLOG_DEBUG(storescuLogger, DIMSE_dumpMessage(str, *req, DIMSE_OUTGOING /*, NULL, presID */));
   }
 
   // We can't use oflog for the pdu output, but we use a special logger for
@@ -1340,16 +1337,18 @@ storeSCU(T_ASC_Association *assoc, const char *fname)
     filexfer = EXS_DeflatedLittleEndianExplicit;
   }
 
-  if (filexfer.getXfer() != EXS_Unknown)
+  if (filexfer != EXS_Unknown)
     presID = ASC_findAcceptedPresentationContextID(assoc, sopClass, filexfer.getXferID());
   else
     presID = ASC_findAcceptedPresentationContextID(assoc, sopClass);
   if (presID == 0)
   {
-    const char *modalityName = dcmSOPClassUIDToModality(sopClass);
-    if (!modalityName) modalityName = dcmFindNameOfUID(sopClass);
-    if (!modalityName) modalityName = "unknown SOP class";
-    OFLOG_ERROR(storescuLogger, "No presentation context for: (" << modalityName << ") " << sopClass);
+    if (storescuLogger.isEnabledFor(OFLogger::ERROR_LOG_LEVEL))
+    {
+      const char *modalityName = dcmSOPClassUIDToModality(sopClass);
+      if (!modalityName) modalityName = dcmFindNameOfUID(sopClass, "unknown SOP class");
+      OFLOG_ERROR(storescuLogger, "No presentation context for: (" << modalityName << ") " << sopClass);
+    }
     renameFile(fname, ".bad");
     return DIMSE_NOVALIDPRESENTATIONCONTEXTID;
   }
@@ -1359,10 +1358,11 @@ storeSCU(T_ASC_Association *assoc, const char *fname)
   DcmXfer netTransfer(pc.acceptedTransferSyntax);
 
   /* if required, dump general information concerning transfer syntaxes */
-  if (storescuLogger.isEnabledFor(OFLogger::INFO_LOG_LEVEL))
+  if ((netTransfer != dcmff.getDataset()->getOriginalXfer()) &&
+    storescuLogger.isEnabledFor(OFLogger::INFO_LOG_LEVEL))
   {
-    DcmXfer fileTransfer(dcmff.getDataset()->getOriginalXfer());
-    OFLOG_INFO(storescuLogger, "Converting transfer syntax: " << fileTransfer.getXferName()
+    DcmXfer origTransfer(dcmff.getDataset()->getOriginalXfer());
+    OFLOG_INFO(storescuLogger, "Converting transfer syntax: " << origTransfer.getXferName()
       << " -> " << netTransfer.getXferName());
   }
 
@@ -1377,7 +1377,7 @@ storeSCU(T_ASC_Association *assoc, const char *fname)
 #endif
 
   /* prepare the transmission of data */
-  bzero(OFreinterpret_cast(char *, &req), sizeof(req));
+  memset(OFreinterpret_cast(char *, &req), 0, sizeof(req));
   req.MessageID = msgId;
   OFStandard::strlcpy(req.AffectedSOPClassUID, sopClass, sizeof(req.AffectedSOPClassUID));
   OFStandard::strlcpy(req.AffectedSOPInstanceUID, sopInstance, sizeof(req.AffectedSOPInstanceUID));
@@ -1387,6 +1387,14 @@ storeSCU(T_ASC_Association *assoc, const char *fname)
   /* if required, dump some more general information */
   OFLOG_INFO(storescuLogger, "Sending Store Request (MsgID " << msgId << ", "
     << dcmSOPClassUIDToModality(sopClass, "OT") << ")");
+  /* the following is needed, because the presentation context ID is not shown otherwise */
+  if (storescuLogger.isEnabledFor(OFLogger::DEBUG_LOG_LEVEL))
+  {
+    const char *sopClassName = dcmFindKeywordOfUID(sopClass, sopClass /*defaultValue*/);
+    const char *netXferName = dcmFindKeywordOfUID(netTransfer.getXferID(), netTransfer.getXferID() /*defaultValue*/);
+    OFLOG_DEBUG(storescuLogger, "  using Presentation Context ID " << OFstatic_cast(int, presID)
+      << " = " << sopClassName << " with " << netXferName << " transfer syntax");
+  }
 
   /* finally conduct transmission of data */
   cond = DIMSE_storeUser(assoc, presID, &req,
