@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2009-2020, OFFIS e.V.
+ *  Copyright (C) 2009-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -98,7 +98,7 @@ OFCondition DcmSCP::openListenPort()
 #ifndef DISABLE_PORT_PERMISSION_CHECK
 #ifdef HAVE_GETEUID
     // If port is privileged we must be as well.
-    if (m_cfg->getPort() < 1024 && geteuid() != 0)
+    if ( ( (m_cfg->getPort() < 1024) && m_cfg->getPort() !=0) && geteuid() != 0)
     {
         DCMNET_ERROR("No privileges to open this network port (" << m_cfg->getPort() << ")");
         return NET_EC_InsufficientPortPrivileges;
@@ -113,13 +113,20 @@ OFCondition DcmSCP::openListenPort()
         return result;
 
     // Initialize network, i.e. create an instance of T_ASC_Network*.
-    cond = ASC_initializeNetwork(NET_ACCEPTOR, OFstatic_cast(int, m_cfg->getPort()), m_cfg->getACSETimeout(), &m_network);
+    const int port = OFstatic_cast(int, m_cfg->getPort());
+    cond = ASC_initializeNetwork(NET_ACCEPTOR, port, m_cfg->getACSETimeout(), &m_network);
     if (cond.bad())
     {
         m_network = NULL;
         return cond;
     }
 
+    // Update config with assigned port if client requested it
+    if (port == 0)
+    {
+      m_cfg->setPort(OFstatic_cast(Uint16, m_network->acceptorPort));
+    }
+
     if (m_cfg->transportLayerEnabled())
     {
       cond = ASC_setTransportLayer(m_network, m_cfg->getTransportLayer(), OFFalse /* Do not take over ownership */);
@@ -853,7 +860,7 @@ OFCondition DcmSCP::sendSTOREResponse(const T_ASC_PresentationContextID presID,
     // Send back response
     T_DIMSE_Message response;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&response, sizeof(response));
+    memset((char*)&response, 0, sizeof(response));
     T_DIMSE_C_StoreRSP& storeRsp       = response.msg.CStoreRSP;
     response.CommandField              = DIMSE_C_STORE_RSP;
     storeRsp.MessageIDBeingRespondedTo = messageID;
@@ -968,7 +975,7 @@ OFCondition DcmSCP::sendFINDResponse(const T_ASC_PresentationContextID presID,
     // Send back response
     T_DIMSE_Message response;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&response, sizeof(response));
+    memset((char*)&response, 0, sizeof(response));
     T_DIMSE_C_FindRSP& findRsp        = response.msg.CFindRSP;
     response.CommandField             = DIMSE_C_FIND_RSP;
     findRsp.MessageIDBeingRespondedTo = messageID;
@@ -1090,7 +1097,7 @@ OFCondition DcmSCP::sendMOVEResponse(const T_ASC_PresentationContextID presID,
     // Send back response
     T_DIMSE_Message response;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&response, sizeof(response));
+    memset((char*)&response, 0, sizeof(response));
     T_DIMSE_C_MoveRSP& moveRsp        = response.msg.CMoveRSP;
     response.CommandField             = DIMSE_C_MOVE_RSP;
     moveRsp.MessageIDBeingRespondedTo = messageID;
@@ -1212,7 +1219,7 @@ OFCondition DcmSCP::sendACTIONResponse(const T_ASC_PresentationContextID presID,
     // Send back response
     T_DIMSE_Message response;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&response, sizeof(response));
+    memset((char*)&response, 0, sizeof(response));
     T_DIMSE_N_ActionRSP& actionRsp      = response.msg.NActionRSP;
     response.CommandField               = DIMSE_N_ACTION_RSP;
     actionRsp.MessageIDBeingRespondedTo = messageID;
@@ -1312,7 +1319,7 @@ OFCondition DcmSCP::handleEVENTREPORTRequest(T_DIMSE_N_EventReportRQ& reqMessage
     // Send back response
     T_DIMSE_Message response;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&response, sizeof(response));
+    memset((char*)&response, 0, sizeof(response));
     T_DIMSE_N_EventReportRSP& eventReportRsp = response.msg.NEventReportRSP;
     response.CommandField                    = DIMSE_N_EVENT_REPORT_RSP;
     eventReportRsp.MessageIDBeingRespondedTo = reqMessage.MessageID;
@@ -1367,7 +1374,7 @@ OFCondition DcmSCP::sendEVENTREPORTRequest(const T_ASC_PresentationContextID pre
     T_ASC_PresentationContextID pcid = presID;
     T_DIMSE_Message request;
     // Make sure everything is zeroed (especially options)
-    bzero((char*)&request, sizeof(request));
+    memset((char*)&request, 0, sizeof(request));
     T_DIMSE_N_EventReportRQ& eventReportReq = request.msg.NEventReportRQ;
     DcmDataset* statusDetail                = NULL;
 
@@ -1453,7 +1460,7 @@ OFCondition DcmSCP::sendEVENTREPORTRequest(const T_ASC_PresentationContextID pre
     rspStatusCode                            = eventReportRsp.DimseStatus;
 
     // Check whether there is a dataset to be received
-    if (eventReportRsp.DataSetType == DIMSE_DATASET_PRESENT)
+    if (eventReportRsp.DataSetType != DIMSE_DATASET_NULL)
     {
         // this should never happen
         DcmDataset* tempDataset = NULL;
@@ -2068,8 +2075,17 @@ void DcmSCP::notifyAssociationRequest(const T_ASC_Parameters& params, DcmSCPActi
 
 // ----------------------------------------------------------------------------
 
-OFBool DcmSCP::checkCalledAETitleAccepted(const OFString& /*calledAETitle*/)
+OFBool DcmSCP::checkCalledAETitleAccepted(const OFString& calledAETitle)
 {
+    if (m_cfg->getRespondWithCalledAETitle() || (calledAETitle == m_cfg->getAETitle()))
+    {
+        return OFTrue;
+    }
+    else if (calledAETitle != m_cfg->getAETitle())
+    {
+        DCMNET_ERROR("Called AE Title does not match configured AE Title: " << m_cfg->getAETitle());
+        return OFFalse;
+    }
     return OFTrue;
 }
 
