@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2012-2017, OFFIS e.V.
+ *  Copyright (C) 2012-2020, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -20,7 +20,6 @@
  */
 
 #include "dcmtk/config/osconfig.h" /* make sure OS specific configuration is included first */
-
 #include "dcmtk/dcmnet/scpcfg.h"
 #include "dcmtk/dcmnet/diutil.h"
 
@@ -40,7 +39,8 @@ DcmSCPConfig::DcmSCPConfig() :
   m_verbosePCMode(OFFalse),
   m_connectionTimeout(1000),
   m_respondWithCalledAETitle(OFTrue),
-  m_progressNotificationMode(OFTrue)
+  m_progressNotificationMode(OFTrue),
+  m_tLayer(NULL)
 {
 }
 
@@ -297,6 +297,27 @@ OFBool DcmSCPConfig::getProgressNotificationMode() const
 
 // ----------------------------------------------------------------------------
 
+OFBool DcmSCPConfig::transportLayerEnabled() const
+{
+  return (m_tLayer != NULL);
+}
+
+// ----------------------------------------------------------------------------
+
+DcmTransportLayer * DcmSCPConfig::getTransportLayer() const
+{
+    return m_tLayer;
+}
+
+// ----------------------------------------------------------------------------
+
+void DcmSCPConfig::setTransportLayer(DcmTransportLayer *tlayer)
+{
+  m_tLayer = tlayer;
+}
+
+// ----------------------------------------------------------------------------
+
 // Reads association configuration from config file
 OFCondition DcmSCPConfig::loadAssociationCfgFile(const OFString &assocFile)
 {
@@ -307,7 +328,7 @@ OFCondition DcmSCPConfig::loadAssociationCfgFile(const OFString &assocFile)
   {
     OFString profileName;
     DCMNET_DEBUG("Loading association configuration file: " << assocFile);
-    result = DcmAssociationConfigurationFile::initialize(m_assocConfig, assocFile.c_str());
+    result = DcmAssociationConfigurationFile::initialize(m_assocConfig, assocFile.c_str(), OFFalse);
     if (result.bad())
     {
       DCMNET_ERROR("Unable to parse association configuration file: " << assocFile << ": " << result.text());
@@ -425,13 +446,13 @@ OFCondition DcmSCPConfig::addPresentationContext(const OFString &abstractSyntax,
     if ( pEntry == NULL)
     {
       // finally add new presentation context to list and profile to configuration
-      if ( result.good() ) result = m_assocConfig.addPresentationContext(DCMSCP_PC_KEY.c_str(), abstractSyntax.c_str(), DCMSCP_TS_KEY.c_str());
+      if ( result.good() ) result = m_assocConfig.addPresentationContext(DCMSCP_PC_KEY.c_str(), abstractSyntax.c_str(), DCMSCP_TS_KEY.c_str(), OFFalse);
       if ( result.good() ) result = m_assocConfig.addProfile(profileName.c_str(), DCMSCP_PC_KEY.c_str(), DCMSCP_RO_KEY.c_str());
     }
     else
     {
       // finally, add presentation context to existing profile
-      result = m_assocConfig.addPresentationContext(pEntry->getPresentationContextKey(), abstractSyntax.c_str(), DCMSCP_TS_KEY.c_str());
+      result = m_assocConfig.addPresentationContext(pEntry->getPresentationContextKey(), abstractSyntax.c_str(), DCMSCP_TS_KEY.c_str(), OFFalse);
     }
   }
   return result;
