@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2000-2014, OFFIS e.V.
+ *  Copyright (C) 2000-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -62,7 +62,7 @@ class DCMTK_DCMSR_EXPORT DSRGraphicDataItem
 
     /** comparison operator
      ** @param  item  item with which the (column,row) pair should be compared
-     ** @return true if not equal, false if equal
+     ** @return OFTrue if not equal, OFFalse if equal
      */
     inline OFBool operator!=(const DSRGraphicDataItem &item) const
     {
@@ -71,7 +71,7 @@ class DCMTK_DCMSR_EXPORT DSRGraphicDataItem
 
     /** comparison operator
      ** @param  item  item with which the (column,row) pair should be compared
-     ** @return true if smaller. Row takes precedence over column.
+     ** @return OFTrue if smaller. Row takes precedence over column.
      */
     inline OFBool operator<(const DSRGraphicDataItem &item) const
     {
@@ -80,7 +80,7 @@ class DCMTK_DCMSR_EXPORT DSRGraphicDataItem
 
     /** comparison operator
      ** @param  item  item with which the (column,row) pair should be compared
-     ** @return true if greater. Row takes precedence over column.
+     ** @return OFTrue if greater. Row takes precedence over column.
      */
     inline OFBool operator>(const DSRGraphicDataItem &item) const
     {
@@ -138,9 +138,11 @@ class DCMTK_DCMSR_EXPORT DSRGraphicDataList
 
     /** read list of graphic data
      ** @param  dataset  DICOM dataset from which the list should be read
+     *  @param  flags    flag used to customize the reading process (see DSRTypes::RF_xxx)
      ** @return status, EC_Normal if successful, an error code otherwise
      */
-    OFCondition read(DcmItem &dataset);
+    OFCondition read(DcmItem &dataset,
+                     const size_t flags);
 
     /** write list of graphic data
      ** @param  dataset  DICOM dataset to which the list should be written
