@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2000-2019, OFFIS e.V.
+ *  Copyright (C) 2000-2022, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -69,7 +69,7 @@ DSRBasicCodedEntry::DSRBasicCodedEntry(const OFString &codeValue,
 // implementation of class DSRCodedEntryValue
 
 DSRCodedEntryValue::DSRCodedEntryValue()
-  : CodeValueType(DSRTypes::CVT_auto),
+  : CodeValueType(DSRTypes::CVT_Short /* should not be "auto" */),
     CodeValue(),
     CodingSchemeDesignator(),
     CodingSchemeVersion(),
@@ -81,6 +81,7 @@ DSRCodedEntryValue::DSRCodedEntryValue()
     ContextGroupLocalVersion(),
     ContextGroupExtensionCreatorUID()
 {
+    /* empty coded entry */
 }
 
 
@@ -226,7 +227,7 @@ OFBool DSRCodedEntryValue::operator!=(const DSRBasicCodedEntry &basicCodedEntry)
 
 void DSRCodedEntryValue::clear()
 {
-    CodeValueType = DSRTypes::CVT_auto;
+    CodeValueType = DSRTypes::CVT_Short /* should not be "auto" */;
     CodeValue.clear();
     CodingSchemeDesignator.clear();
     CodingSchemeVersion.clear();
@@ -768,12 +769,20 @@ OFCondition DSRCodedEntryValue::checkCode(const OFString &codeValue,
 {
     OFCondition result = EC_Normal;
     /* first, make sure that the mandatory values are non-empty and the type is valid */
-    if (codeValue.empty() || (codingSchemeDesignator.empty() && (codeValueType != DSRTypes::CVT_URN)) || codeMeaning.empty())
-        result = SR_EC_InvalidValue;
+    if (codeValueType == DSRTypes::CVT_URN)
+    {
+        /* CP-1913: Coding Scheme Version shall not be present if Coding Scheme Designator is absent */
+        if (codeValue.empty() || (codingSchemeDesignator.empty() && !codingSchemeVersion.empty()) || codeMeaning.empty())
+            result = SR_EC_InvalidValue;
+    }
     else if (codeValueType == DSRTypes::CVT_auto)
     {
         DCMSR_DEBUG("INTERNAL ERROR: DSRCodedEntryValue::checkCode() called with DSRTypes::CVT_auto");
         result = EC_IllegalCall;
+    } else {
+        /* short or long code value */
+        if (codeValue.empty() || codingSchemeDesignator.empty() || codeMeaning.empty())
+            result = SR_EC_InvalidValue;
     }
     /* then, check whether the passed values are valid with regards to VR and VM.
      * tbd: unfortunately, we do not know the character set, so "UNKNOWN" is used. */
