@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2010-2015, OFFIS e.V.
+ *  Copyright (C) 2010-2021, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -74,24 +74,29 @@ OFBool DSRColonCadSRConstraintChecker::checkContentRelationship(const E_ValueTyp
     /* row 1 of the table */
     if ((relationshipType == RT_contains) && !byReference && (sourceValueType == VT_Container))
     {
-        result = (targetValueType == VT_Code) || (targetValueType == VT_Num) || (targetValueType == VT_Image) ||
-                 (targetValueType == VT_Container) || (targetValueType == VT_UIDRef) || (targetValueType == VT_Date) ||
+        result = (targetValueType == VT_Code)      || (targetValueType == VT_Num)    || (targetValueType == VT_Image) ||
+                 (targetValueType == VT_Container) || (targetValueType == VT_UIDRef) || (targetValueType == VT_Date)  ||
                  (targetValueType == VT_Time);
     }
     /* row 2 of the table */
     else if ((relationshipType == RT_hasObsContext) && !byReference && ((sourceValueType == VT_Container) ||
         (sourceValueType == VT_Text) || (sourceValueType == VT_Code) || (sourceValueType == VT_Num)))
     {
-        result = (targetValueType == VT_Text) || (targetValueType == VT_Code) || (targetValueType == VT_Num) ||
-                 (targetValueType == VT_Date) || (targetValueType == VT_Time) || (targetValueType == VT_PName) ||
+        result = (targetValueType == VT_Text)   || (targetValueType == VT_Code) || (targetValueType == VT_Num)   ||
+                 (targetValueType == VT_Date)   || (targetValueType == VT_Time) || (targetValueType == VT_PName) ||
                  (targetValueType == VT_UIDRef) || (targetValueType == VT_Composite);
     }
+    /* new row introduced with CP-2084 */
+    else if ((relationshipType == RT_hasObsContext) && (sourceValueType == VT_Container))
+    {
+        result = (targetValueType == VT_Container);
+    }
     /* row 3 of the table */
     else if ((relationshipType == RT_hasAcqContext) && (sourceValueType == VT_Image))
     {
         /* by-reference allowed */
         result = (targetValueType == VT_Text) || (targetValueType == VT_Code) || (targetValueType == VT_Date) ||
-                 (targetValueType == VT_Time) || (targetValueType == VT_Num) || (targetValueType == VT_Container);
+                 (targetValueType == VT_Time) || (targetValueType == VT_Num)  || (targetValueType == VT_Container);
     }
     /* row 4 of the table */
     else if ((relationshipType == RT_hasConceptMod) && !byReference && ((sourceValueType == VT_Container) ||
@@ -103,15 +108,15 @@ OFBool DSRColonCadSRConstraintChecker::checkContentRelationship(const E_ValueTyp
     else if ((relationshipType == RT_hasProperties) && !byReference &&
         ((sourceValueType == VT_Text) || (sourceValueType == VT_Code) || (sourceValueType == VT_Num)))
     {
-        result = (targetValueType == VT_Container) || (targetValueType == VT_Text) || (targetValueType == VT_Code) ||
-                 (targetValueType == VT_Num) || (targetValueType == VT_Date) || (targetValueType == VT_Image) ||
-                 (targetValueType == VT_SCoord) || (targetValueType == VT_SCoord3D) || (targetValueType == VT_UIDRef);
+        result = (targetValueType == VT_Container) || (targetValueType == VT_Text)     || (targetValueType == VT_Code)  ||
+                 (targetValueType == VT_Num)       || (targetValueType == VT_Date)     || (targetValueType == VT_Image) ||
+                 (targetValueType == VT_SCoord)    || (targetValueType == VT_SCoord3D) || (targetValueType == VT_UIDRef);
     }
     /* row 6 of the table */
     else if ((relationshipType == RT_inferredFrom) && ((sourceValueType == VT_Code) || (sourceValueType == VT_Num)))
     {
         /* by-reference allowed */
-        result = (targetValueType == VT_Code) || (targetValueType == VT_Num) || (targetValueType == VT_Image) ||
+        result = (targetValueType == VT_Code)   || (targetValueType == VT_Num)      || (targetValueType == VT_Image)     ||
                  (targetValueType == VT_SCoord) || (targetValueType == VT_SCoord3D) || (targetValueType == VT_Container) ||
                  (targetValueType == VT_Text);
     }
