@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2000-2018, OFFIS e.V.
+ *  Copyright (C) 2000-2019, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -155,7 +155,7 @@ OFCondition DSRImageTreeNode::readXMLContentItem(const DSRXMLDocument &doc,
                                                  const size_t flags)
 {
     /* retrieve value from XML element "value" */
-    return DSRImageReferenceValue::readXML(doc, doc.getNamedNode(cursor.gotoChild(), "value"), flags);
+    return DSRImageReferenceValue::readXML(doc, doc.getNamedChildNode(cursor, "value"), flags);
 }
 
 
