@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2011-2019, OFFIS e.V.
+ *  Copyright (C) 2011-2023, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -26,9 +26,7 @@
 #include "dcmtk/ofstd/ofstd.h"
 #include "dcmtk/ofstd/ofdiag.h"
 #include "dcmtk/ofstd/ofconsol.h"
-
-#define INCLUDE_CERRNO
-#include "dcmtk/ofstd/ofstdinc.h"
+#include <cerrno>
 
 #ifdef HAVE_WINDOWS_H
 #define WIN32_LEAN_AND_MEAN
@@ -293,12 +291,35 @@ class OFCharacterEncoding::Implementation
 };
 
 #elif DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_ICONV ||\
+ DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV ||\
  DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_STDLIBC_ICONV
 
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+
+#include "dcmtk/oficonv/iconv.h"
+
+#undef LIBICONV_SECOND_ARGUMENT_CONST
+#define iconv_open OFiconv_open
+#define iconv_close OFiconv_close
+#define iconv OFiconv
+#define iconvctl OFiconvctl
+
+#ifndef WITH_LIBICONV
+#define WITH_LIBICONV
+#endif
+
+#else /* DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV */
+
 #include <iconv.h>
 #ifdef WITH_LIBICONV
 #include <localcharset.h>
 #endif
+#ifdef __GLIBC__
+#include <langinfo.h> // nl_langinfo / CODESET
+#endif
+
+#endif /* DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV */
+
 
 #define ILLEGAL_DESCRIPTOR     OFreinterpret_cast(iconv_t, -1)
 #define CONVERSION_ERROR       OFstatic_cast(size_t, -1)
@@ -320,6 +341,11 @@ class OFCharacterEncoding::Implementation
             createErrnoCondition(result, "Cannot open character encoding: ", EC_CODE_CannotOpenEncoding);
             return OFnullptr;
         }
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+        // GNU libiconv does not perform transliteration by default. We emulate this when using oficonv.
+        int flag1 = 1;
+        (void) ::iconvctl(descriptor, ICONV_SET_ILSEQ_INVALID, &flag1);
+#endif
         if (Implementation* pImplementation = new Implementation(descriptor))
         {
             result = EC_Normal;
@@ -331,7 +357,13 @@ class OFCharacterEncoding::Implementation
 
     static OFString getVersionString()
     {
-#ifdef WITH_LIBICONV
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+        return "Citrus iconv, Version FreeBSD 13.1 (modified)"
+#ifdef DCMTK_ENABLE_BUILTIN_OFICONV_DATA
+            ", with built-in data files"
+#endif
+            ;
+#elif defined(WITH_LIBICONV)
         OFString versionStr = "LIBICONV, Version ";
         char buf[10];
         // extract major and minor version number
@@ -352,10 +384,18 @@ class OFCharacterEncoding::Implementation
 
     static OFString getLocaleEncoding()
     {
-#ifdef WITH_LIBICONV
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+        iconv_locale_allocation_t buf;
+        return OFSTRING_GUARD(::OFlocale_charset(&buf));
+#elif defined(WITH_LIBICONV)
         // basically, the function below should always return a non-empty string
         // but older versions of libiconv might return NULL in certain cases
         return OFSTRING_GUARD(::locale_charset());
+#elif defined(__GLIBC__)
+        const char *oldlocale = setlocale(LC_ALL, "");
+        const char *codeset = nl_langinfo (CODESET);
+        setlocale(LC_ALL, oldlocale);
+        return OFSTRING_GUARD(codeset);
 #else
         return OFString();
 #endif
@@ -364,11 +404,11 @@ class OFCharacterEncoding::Implementation
     static OFBool supportsConversionFlags(const unsigned flags)
     {
 #if defined(WITH_LIBICONV) && _LIBICONV_VERSION >= 0x0108
-        return flags == AbortTranscodingOnIllegalSequence
+        return  flags == AbortTranscodingOnIllegalSequence
             || flags == DiscardIllegalSequences
             || flags == TransliterateIllegalSequences
             || flags == (DiscardIllegalSequences | TransliterateIllegalSequences)
-        ;
+            ;
 #elif defined DCMTK_FIXED_ICONV_CONVERSION_FLAGS
         // the iconvctl function is implemented only in newer versions of the
         // GNU libiconv and not in other iconv implementations. For instance,
@@ -376,8 +416,9 @@ class OFCharacterEncoding::Implementation
         // different encoding flags and only has a (varying) fixed functionality
         // that we detect with a configuration test.
         return flags == DCMTK_FIXED_ICONV_CONVERSION_FLAGS;
-#endif
+#else
         return OFFalse;
+#endif
     }
 
     unsigned getConversionFlags() const
@@ -385,10 +426,17 @@ class OFCharacterEncoding::Implementation
 #if defined(WITH_LIBICONV) && _LIBICONV_VERSION >= 0x0108
         unsigned result = 0;
         int flag;
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+        if (::iconvctl(ConversionDescriptor, ICONV_GET_ILSEQ_INVALID, &flag))
+            return 0;
+        if (flag == 0)
+            result |= TransliterateIllegalSequences;
+#else
         if (::iconvctl(ConversionDescriptor, ICONV_GET_TRANSLITERATE, &flag))
             return 0;
         if (flag)
             result |= TransliterateIllegalSequences;
+#endif
         if (::iconvctl(ConversionDescriptor, ICONV_GET_DISCARD_ILSEQ, &flag))
             return 0;
         if (flag)
@@ -404,42 +452,69 @@ class OFCharacterEncoding::Implementation
         // different encoding flags and only has a (varying) fixed functionality
         // that we detect with a configuration test.
         return DCMTK_FIXED_ICONV_CONVERSION_FLAGS;
-#endif
+#else
         return 0;
+#endif
     }
 
     OFBool setConversionFlags(const unsigned flags)
     {
 #if defined(WITH_LIBICONV) && _LIBICONV_VERSION >= 0x0108
-        int flag = 0;
+        int flag0 = 0;
+        int flag1 = 1;
         switch (flags)
         {
             case AbortTranscodingOnIllegalSequence:
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag))
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag0))
                     return OFFalse;
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag))
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_ILSEQ_INVALID, &flag1))
                     return OFFalse;
+#else
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag0))
+                    return OFFalse;
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag0))
+                    return OFFalse;
+#endif
                 return OFTrue;
             case DiscardIllegalSequences:
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag))
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag1))
                     return OFFalse;
-                flag = 1;
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag))
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_ILSEQ_INVALID, &flag0))
                     return OFFalse;
+#else
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag1))
+                    return OFFalse;
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag0))
+                    return OFFalse;
+#endif
                 return OFTrue;
             case TransliterateIllegalSequences:
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag))
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag0))
                     return OFFalse;
-                flag = 1;
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag))
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_ILSEQ_INVALID, &flag0))
                     return OFFalse;
+#else
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag0))
+                    return OFFalse;
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag1))
+                    return OFFalse;
+#endif
                 return OFTrue;
             case (TransliterateIllegalSequences | DiscardIllegalSequences):
-                flag = 1;
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag))
+#if DCMTK_ENABLE_CHARSET_CONVERSION == DCMTK_CHARSET_CONVERSION_OFICONV
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag1))
                     return OFFalse;
-                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag))
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_ILSEQ_INVALID, &flag0))
                     return OFFalse;
+#else
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_DISCARD_ILSEQ, &flag1))
+                    return OFFalse;
+                if (::iconvctl(ConversionDescriptor, ICONV_SET_TRANSLITERATE, &flag1))
+                    return OFFalse;
+#endif
                 return OFTrue;
             default:
                 return OFFalse;
@@ -451,8 +526,9 @@ class OFCharacterEncoding::Implementation
         // different encoding flags and only has a (varying) fixed functionality
         // that we detect with a configuration test.
         return flags == DCMTK_FIXED_ICONV_CONVERSION_FLAGS;
-#endif
+#else
         return OFFalse;
+#endif
     }
 
 
@@ -600,14 +676,17 @@ OFString OFCharacterEncoding::getLocaleEncoding()
 }
 
 
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 OFBool OFCharacterEncoding::supportsConversionFlags(const unsigned flags)
 {
-#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
     return Implementation::supportsConversionFlags(flags);
-#else
-    return OFFalse;
-#endif
 }
+#else
+OFBool OFCharacterEncoding::supportsConversionFlags(const unsigned /* flags */)
+{
+    return OFFalse;
+}
+#endif
 
 
 OFCharacterEncoding::OFCharacterEncoding()
@@ -678,9 +757,9 @@ unsigned OFCharacterEncoding::getConversionFlags() const
 }
 
 
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 OFCondition OFCharacterEncoding::setConversionFlags(const unsigned flags)
 {
-#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
     if (TheImplementation)
     {
         if (TheImplementation->setConversionFlags(flags))
@@ -689,42 +768,55 @@ OFCondition OFCharacterEncoding::setConversionFlags(const unsigned flags)
             "Conversion flags not supported by the underlying implementation");
     }
     return EC_NoEncodingSelected;
-#endif
+}
+#else
+OFCondition OFCharacterEncoding::setConversionFlags(const unsigned /* flags */)
+{
     return EC_NoEncodingLibrary;
 }
+#endif
 
 
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 OFCondition OFCharacterEncoding::selectEncoding(const OFString &fromEncoding,
                                                 const OFString &toEncoding)
 {
-#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
     OFCondition result;
     TheImplementation.reset(Implementation::create(fromEncoding, toEncoding, result));
     return result;
-#else
-    return EC_NoEncodingLibrary;
-#endif
 }
+#else
+OFCondition OFCharacterEncoding::selectEncoding(const OFString & /* fromEncoding */,
+                                                const OFString & /* toEncoding */)
+{
+    return EC_NoEncodingLibrary;
+}
+#endif
 
 
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 OFCondition OFCharacterEncoding::convertString(const OFString &fromString,
                                                OFString &toString,
                                                const OFBool clearMode)
 {
-#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
     return convertString(fromString.c_str(), fromString.length(), toString, clearMode);
-#else
-    return EC_NoEncodingLibrary;
-#endif
 }
+#else
+OFCondition OFCharacterEncoding::convertString(const OFString & /* fromString */,
+                                               OFString & /* toString */,
+                                               const OFBool /* clearMode */)
+{
+    return EC_NoEncodingLibrary;
+}
+#endif
 
 
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 OFCondition OFCharacterEncoding::convertString(const char *fromString,
                                                const size_t fromLength,
                                                OFString &toString,
                                                const OFBool clearMode)
 {
-#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
     if (TheImplementation)
     {
         // first, clear result variable if requested
@@ -733,10 +825,16 @@ OFCondition OFCharacterEncoding::convertString(const char *fromString,
         return TheImplementation->convert(toString, fromString, fromLength);
     }
     return EC_NoEncodingSelected;
-#else
-    return EC_NoEncodingLibrary;
-#endif
 }
+#else
+OFCondition OFCharacterEncoding::convertString(const char * /* fromString */,
+                                               const size_t /* fromLength */,
+                                               OFString & /* toString */,
+                                               const OFBool /* clearMode */)
+{
+    return EC_NoEncodingLibrary;
+}
+#endif
 
 
 #ifdef HAVE_WINDOWS_H  // Windows-specific conversion functions
