@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1999-2018, OFFIS e.V.
+ *  Copyright (C) 1999-2022, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -28,9 +28,8 @@
 #ifdef DCMTK_ENABLE_CHARSET_CONVERSION
 #include "dcmtk/ofstd/ofchrenc.h"     /* for OFCharacterEncoding */
 
-#define INCLUDE_LOCALE
-#include "dcmtk/ofstd/ofstdinc.h"     /* for setlocale() */
-#endif // WITH_LIBICONV
+#include <locale>
+#endif // DCMTK_ENABLE_CHARSET_CONVERSION
 
 #ifdef HAVE_WINDOWS_H
 #define WIN32_LEAN_AND_MEAN
@@ -159,7 +158,7 @@ void OFConsoleApplication::printHeader(const OFBool hostInfo,
         }
 #endif
 #ifdef HAVE_WINDOWS_H
-        if ((CmdLine != NULL) && (CmdLine->getWideCharMode()))
+        if ((CmdLine != NULL) && CmdLine->getWideCharMode())
         {
             (*output) << "Character encoding: Unicode (UTF-16)" << OFendl;
         } else {
@@ -167,10 +166,64 @@ void OFConsoleApplication::printHeader(const OFBool hostInfo,
             (*output) << "Code page: " << GetOEMCP() << " (OEM) / " << GetACP() << " (ANSI)" << OFendl;
         }
 #endif
+        /* output details on DCMTK's build options */
+        (*output) << "Build options:";
 #ifdef DEBUG
         /* indicate that debug code is present */
-        (*output) << OFendl << "Compiled with DEBUG defined, i.e. with debug code" << OFendl;
+        (*output) << " debug";
 #endif
+#ifdef ofstd_EXPORTS
+        /* indicate that shared library support is enabled */
+        (*output) << " shared";
+#endif
+#ifdef HAVE_CXX11
+        /* indicate that C++11 is used */
+        (*output) << " cxx11";
+#endif
+#if defined(HAVE_STL_ALGORITHM) && defined(HAVE_STL_LIMITS) && defined(HAVE_STL_LIST) && \
+    defined(HAVE_STL_MAP) && defined(HAVE_STL_MEMORY) && defined(HAVE_STL_STACK) && \
+    defined(HAVE_STL_STRING) && defined(HAVE_STL_SYSTEM_ERROR) && defined(HAVE_STL_TUPLE) && \
+    defined(HAVE_STL_TYPE_TRAITS) && defined(HAVE_STL_VECTOR)
+        /* indicate that the C++ STL is used */
+        (*output) << " stl";
+#elif defined(HAVE_STL_ALGORITHM) || defined(HAVE_STL_LIMITS) || defined(HAVE_STL_LIST) || \
+    defined(HAVE_STL_MAP) || defined(HAVE_STL_MEMORY) || defined(HAVE_STL_STACK) || \
+    defined(HAVE_STL_STRING) || defined(HAVE_STL_SYSTEM_ERROR) || defined(HAVE_STL_TUPLE) || \
+    defined(HAVE_STL_TYPE_TRAITS) || defined(HAVE_STL_VECTOR)
+        /* indicate that parts of the C++ STL are used */
+        (*output) << " (stl)";
+#endif
+#ifdef WITH_THREADS
+        /* indicate that MT support is enabled */
+        (*output) << " threads";
+#endif
+#ifdef DCMTK_ENABLE_LFS
+        /* indicate that LFS support is enabled */
+        (*output) << " lfs";
+#endif
+#if DCM_DICT_DEFAULT == 1
+        /* indicate that the builtin data dictionary is enabled */
+        (*output) << " builtin-dict";
+#elif DCM_DICT_DEFAULT == 2
+        /* indicate that the external data dictionary is enabled */
+        (*output) << " extern-dict";
+#endif
+#ifdef DCM_DICT_USE_DCMDICTPATH
+        /* indicate that the DCMDICTPATH environment variable is checked */
+        (*output) << " dcmdictpath";
+#elif DCM_DICT_DEFAULT == 0
+        /* indicate that no data dictionary is available */
+        (*output) << " no-dict";
+#endif
+#ifdef ENABLE_PRIVATE_TAGS
+        /* indicate that private tag dictionary is enabled */
+        (*output) << " private-tags";
+#endif
+#ifdef DCMTK_ENABLE_CHARSET_CONVERSION
+        /* indicate that character set conversion is enabled */
+        (*output) << " char-conv";
+#endif
+        (*output) << OFendl;
     }
     /* release output stream */
     if (stdError)
