@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2011-2019, OFFIS e.V.
+ *  Copyright (C) 2011-2022, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -22,7 +22,13 @@
 #include "dcmtk/config/osconfig.h"
 
 #define OFTEST_OFSTD_ONLY
+
+#ifdef _WIN32
+#define OFTEST_INCLUDE_IPC_OPTION
+#endif
+
 #include "dcmtk/ofstd/oftest.h"
+#include "dcmtk/ofstd/ofxml.h"
 
 OFTEST_REGISTER(ofstd_OFCharacterEncoding_1);
 OFTEST_REGISTER(ofstd_OFCharacterEncoding_2);
@@ -72,7 +78,9 @@ OFTEST_REGISTER(ofstd_testPaths_2);
 #ifdef WITH_THREADS
 OFTEST_REGISTER(ofstd_thread);
 #endif // WITH_THREADS
+#ifndef _XMLWIDECHAR
 OFTEST_REGISTER(ofstd_xmlParser);
+#endif
 OFTEST_REGISTER(ofstd_memory);
 OFTEST_REGISTER(ofstd_optional);
 OFTEST_REGISTER(ofstd_tuple);
@@ -83,5 +91,7 @@ OFTEST_REGISTER(ofstd_error);
 OFTEST_REGISTER(ofstd_filesystem);
 OFTEST_REGISTER(ofstd_snprintf);
 OFTEST_REGISTER(ofstd_OFStringUtil_replace_all);
+OFTEST_REGISTER(ofstd_gethostnamebyaddress);
+OFTEST_REGISTER(ofstd_ipc);
 
 OFTEST_MAIN("ofstd")
